#!/bin/bash

. /usr/share/univention-lib/ucr.sh

eval "$(ucr shell samba4/role)"

check_samba_processes_ucs5() {
	local samba_service_names
	local pids
	local samba_processes
	local min_samba_processes

	rc=0
	samba_service_names=(samba cldap dnsupdate drepl kcc kdc ldap smbd-notifyd 'rpc[\[(]' winbind nmbd)
	for srv in "${samba_service_names[@]}"; do
		if [ "$srv" = "drepl" ] && is_ucr_false samba4/service/drepl; then
			continue
		fi
		pgrep -l "^$srv"
		pids=($(pgrep "^$srv"))
		if [ "${#pids[@]}" = 0 ]; then
			echo "ERROR: Samba service '$srv' not found in process list"
			rc=1
		fi
	done
	if [ "$rc" != 0 ]; then
		return "$rc"
	fi
}

check_samba_processes_ucs4() {
	local samba_pids
	local comm
	local samba_process_name
	local pid
	local samba_processes
	local min_samba_processes

	rc=0
	samba_pids=$(samba-tool processes 2>/dev/null | tail -n +3 | grep -v winbind_server | sed -rn 's/.*\<([0-9]+)\>$/\1/p' | grep -v '^0$')
	for pid in $samba_pids; do

		# sometimes samba-tool processes returns a pid like 18446744073709551615
		# which is obviously wrong, so ignore those pids
		test ${#pid} -gt 10 && continue

		comm=$(ps -p "$pid" -o comm=)
		if [ -z "$comm" ]; then
			samba_process_name=$(samba-tool processes --pid="$pid")
			if [ -n "$samba_process_name" ]; then
				echo "ERROR: This Samba process $pid is missing: $samba_process_name"
				rc=1
			fi
		fi
	done
	if [ "$rc" != 0 ]; then
		return "$rc"
	fi

	samba_processes="$(pidof -c /usr/sbin/samba | wc -w)"
	min_samba_processes=12
	if [ "$samba_processes" -lt "$min_samba_processes" ]; then
		echo "WARNING: Total number of /usr/sbin/samba processes lower than $min_samba_processes"
		echo "This might be ok when the Samba release is newer than this check tool."
	fi
}

if [ "$samba4_role" = "DC" ]; then
	if dpkg --compare-versions "$(dpkg-query -f '${Version}' -W samba)" gt 2:4.13.4; then
		check_samba_processes_ucs5
	else
		check_samba_processes_ucs4
	fi
fi
